package pomodoro

import (
	"git.smsvc.net/pomodoro/GoTomato/internal/broadcast"
	"git.smsvc.net/pomodoro/GoTomato/pkg/models"
	"github.com/gorilla/websocket"
	"sync"
)

const (
	workDuration       = 15 * 60
	shortBreakDuration = 5 * 60
	longBreakDuration  = 10 * 60
	sessions           = 4
)

var pomodoroRunning bool
var pomodoroPaused bool

var pomodoroResetChannel = make(chan bool, 1)
var pomodoroPauseChannel = make(chan bool, 1)
var pomodoroResumeChannel = make(chan bool, 1)

var mu sync.Mutex // to synchronize access to shared state

// RunPomodoroTimer iterates the Pomodoro work/break sessions.
func RunPomodoroTimer(clients map[*websocket.Conn]bool) {
	mu.Lock()
	pomodoroRunning = true
	pomodoroPaused = false
	mu.Unlock()

	for session := 1; session <= sessions; session++ {
		if !startTimer(clients, workDuration, "Work", session) {
			break
		}
		if session == sessions {
			if !startTimer(clients, longBreakDuration, "LongBreak", session) {
				break
			}
		} else {
			if !startTimer(clients, shortBreakDuration, "ShortBreak", session) {
				break
			}
		}
	}

	mu.Lock()
	pomodoroRunning = false
	mu.Unlock()
}

// ResetPomodoro resets the running Pomodoro timer.
func ResetPomodoro(clients map[*websocket.Conn]bool) {
	mu.Lock()
	pomodoroRunning = false // Reset the running state
	pomodoroPaused = false  // Reset the paused state
	mu.Unlock()

	// Broadcast the reset message to all clients
	broadcast.BroadcastMessage(clients, models.BroadcastMessage{
		Mode:       "none",
		Session:    0,
		MaxSession: 0,
		TimeLeft:   0,
	})

	// Send a reset signal to stop any running timers
	pomodoroResetChannel <- true
}

func PausePomodoro() {
	pomodoroPauseChannel <- true
}

func ResumePomodoro() {
	pomodoroResumeChannel <- true
}

func IsPomodoroRunning() bool {
	mu.Lock()
	defer mu.Unlock() // Ensures that the mutex is unlocked after the function is done
	return pomodoroRunning
}

func IsPomodoroPaused() bool {
	mu.Lock()
	defer mu.Unlock() // Ensures that the mutex is unlocked after the function is done
	return pomodoroPaused
}
